import shutil
import tempfile
import urllib
import subprocess
import os
import numpy as np
from datetime import datetime


class PerformanceTest(object):
    name = 'void'

    def __init__(self, scratch_folder, origin):
        self.origin = origin
        self.scratch_folder = scratch_folder
        self.scratch_folder = tempfile.mkdtemp(dir=self.scratch_folder)
        self.destiny = os.path.join(scratch_folder, '{}.nc'.format(self.name))
        self.times = []

    def make_test(self):
        print 'Launching {0} test'.format(self.name)
        if os.path.isfile(self.destiny):
            os.remove(self.destiny)
        start = datetime.now()
        self.get_data()
        ellapsed = datetime.now() - start
        self.times.append(ellapsed.total_seconds())
        print 'Tested {0}: {1}'.format(self.name, ellapsed)
        os.remove(self.destiny)

    def get_data(self):
        pass

    def compute_results(self):
        self.time = np.array(self.times)
        self.average = np.average(self.time)
        self.std = np.std(self.time)
        self.sum = np.sum(self.time)

    def clean(self):
        shutil.rmtree(self.scratch_folder)


class DodscTest(PerformanceTest):
    name = 'DodsC'
    color = 'g'

    def get_data(self):
        subprocess.check_call(['nccopy', '-4', self.origin, self.destiny])


class FileServerTest(PerformanceTest):
    name = 'FileServer'
    color = 'b'

    def get_data(self):
        urllib.urlretrieve(self.origin, self.destiny)


class CopyTest(PerformanceTest):
    name = 'Copy'
    color = 'r'

    def get_data(self):
        shutil.copyfile(self.origin, self.destiny)


class DirectRetrieveTest(PerformanceTest):
    name = 'DirectRetrieve'
    color = 'y'

    def get_data(self):
        urllib.urlretrieve(self.origin, self.destiny)


class NcksTest(PerformanceTest):
    name = 'ncks'
    color = '#FFA500'

    def get_data(self):
        subprocess.check_call('ncks -v sb -d x,0,1000 {0} -o {1}'.format(self.origin, self.destiny).split())


class SubsetTest(PerformanceTest):
    name = 'Subset'
    color = '#DA70D6'

    def get_data(self):
        subprocess.check_call(['nccopy', '-4', '{0}?sb[0:1:0][0:1:74][0:1:1020][0:1:1441]'.format(self.origin),
                               self.destiny])



